@@ -20,7 +20,7 @@ logilab.devtools packaging information
 
 modname = 'devtools'
 distname = 'logilab-devtools'
-numversion = (0, 13, 1)
+numversion = (0, 15, 0)
 version = '.'.join([str(num) for num in numversion])
 
 license = 'GPL'
@@ -37,7 +37,7 @@ long_desc = """Set of tools which aims to help the developpement process, includ
 * python coverage tool
 * cvs/svn utilities
 """
-web = "http://www.logilab.org/projects/devtools"
+web = "http://www.logilab.org/project/devtools"
 ftp = "ftp://ftp.logilab.org/pub/devtools"
 mailinglist = "mailto://python-projects@lists.logilab.org"
 
@@ -59,7 +59,7 @@ scripts = [
     'bin/changelog',
     'bin/update_gettext',
     'bin/lgp',
-    'bin/setupchroot',
     'bin/vbuild',
+    'bin/lsprofcalltree',
     ] 
 
