@@ -30,42 +30,41 @@ __docformat__ = "restructuredtext en"
 
 
 import os
+import sys
 import stat
 import re
 import commands
 import logging
+import subprocess
 from os.path import basename, join, exists, isdir, isfile
-from pprint import pprint
+from pprint import pformat
 
 from logilab.common.compat import set
 from logilab.common.fileutils import ensure_fs_mode
 
-from logilab.devtools.vcslib import get_vcs_agent, BASE_EXCLUDE
-from logilab.devtools.lib.pkginfo import PackageInfo
 from logilab.devtools.lib.pkginfo import check_url as _check_url, spell_check, get_default_scripts, sequence_equal
 from logilab.devtools.lib.manifest import (get_manifest_files, read_manifest_in,
                                            match_extensions, JUNK_EXTENSIONS)
 
 from logilab.devtools import templates
-from logilab.devtools.lgp.changelog import ChangeLog, ChangeLogNotFound, \
-     find_ChangeLog, CHANGEFILE
 from logilab.devtools.lgp.setupinfo import SetupInfo
-from logilab.devtools.lgp.utils import get_distributions, get_architectures
 from logilab.devtools.lgp.utils import cond_exec, confirm
 from logilab.devtools.lgp.exceptions import LGPException
 
-
+CHANGEFILE='ChangeLog'
 MANDATORY_SETUP_FIELDS = ('name', 'version', 'author', 'author_email', 'license',
                           'copyright', 'short_desc', 'long_desc')
 
 OK, NOK = 1, 0
 CHECKS = { 'default'    : ['debian_dir', 'debian_rules', 'debian_copying',
+                           'debian_source_value',
                            'debian_changelog', 'package_info', 'readme',
                            'changelog', 'bin', 'tests_directory', 'setup_file',
-                           'repository', 'copying', 'documentation', 'debsign',
+                           'repository', 'copying', 'documentation',
                            'homepage', 'builder', 'keyrings', 'announce',
                            'release_number', 'manifest_in', 'include_dirs',
-                           'scripts', 'pydistutils'],
+                           'scripts', 'pydistutils', 'debian_maintainer',
+                           'debian_uploader'],
            'setuptools' : [],
            'pkginfo'    : [],
            'makefile'   : ['makefile'],
@@ -159,7 +158,7 @@ def run(args):
         checker.start_checks()
 
         # Return the number of invalid tests
-        return len(checker.get_checklist())-checker.counter
+        return checker.errors()
 
     except NotImplementedError, exc:
         logging.error(exc)
@@ -170,7 +169,7 @@ def run(args):
 
 
 class Checker(SetupInfo):
-    """ Package checker class
+    """Lgp checker class
 
     Specific options are added. See lgp check --help
     """
@@ -180,7 +179,6 @@ class Checker(SetupInfo):
     options = (('include',
                 {'type': 'csv',
                  'dest': 'include_checks',
-                 #'default' : [],
                  'short': 'i',
                  'metavar' : "<comma separated names of check functions>",
                  'help': "force the inclusion of other check functions"
@@ -188,7 +186,6 @@ class Checker(SetupInfo):
                ('exclude',
                 {'type': 'csv',
                  'dest': 'exclude_checks',
-                 #'default' : [],
                  'short': 'e',
                  'metavar' : "<comma separated names of check functions>",
                  'help': "force the exclusion of other check functions"
@@ -196,59 +193,61 @@ class Checker(SetupInfo):
                ('set',
                 {'type': 'csv',
                  'dest': 'set_checks',
-                 #'default' : [],
+                 'default' : CHECKS['default'],
                  'short': 's',
                  'metavar' : "<comma separated names of check functions>",
                  'help': "set a specific check functions list"
                 }),
                ('list',
                 {'action': 'store_true',
-                 'default': False,
                  'dest' : "list_checks",
                  'short': 'l',
                  'help': "return a list of all available check functions"
                 }),
-               ('only',
-                {'action': 'store_true',
-                 'default': False,
-                 'dest' : "only_one_check",
-                 'short': 'o',
-                 'help': "run only one single test"
-                }),
               ),
 
     def __init__(self, args):
         # Retrieve upstream information
         super(Checker, self).__init__(arguments=args, options=self.options, usage=__doc__)
-        # We force debian check only for 'unstable' distribution
-        self.config.distrib = 'unstable'
-        self.logger = logging.getLogger(__name__)
+
+    def errors(self):
+        return len(self.get_checklist())-self.counter
 
     def get_checklist(self, all=False):
         if all:
             return [funct for (name, funct) in globals().items() if name.startswith('check_')]
 
-        if self.config.set_checks is not None:
-            checks = []
-            for check in self.config.set_checks:
-                checks.append(check)
-        else:
-            checks = CHECKS['default'] + CHECKS[self._package_format]
-        if self.config.include_checks is not None:
-            for check in self.config.include_checks:
-                checks.append(check)
-        if self.config.exclude_checks is not None:
-            for check in self.config.exclude_checks:
-                checks.remove(check)
-        if self.config.only_one_check:
-            checks = (checks[-1],)
+        assert self.config.set_checks, 'set_checks should never be empty here'
 
-        self.checklist = [globals()["check_%s" % name] for name in checks]
+        try:
+            try:
+                if self.config.set_checks:
+                    checks = []
+                    for check in self.config.set_checks:
+                        checks.append(check)
+                    #self.logger.debug("active checks: %s" % ','.join(checks))
+
+                if self.config.include_checks is not None:
+                    for check in self.config.include_checks:
+                        #self.logger.debug("include check: %s" % check)
+                        checks.append(check)
+                if self.config.exclude_checks is not None:
+                    for check in self.config.exclude_checks:
+                        #self.logger.debug("exclude check: %s" % check)
+                        checks.remove(check)
+                self.checklist = [globals()["check_%s" % name] for name in checks]
+            except ValueError, err:
+                raise KeyError(check)
+        except KeyError, err:
+            raise LGPException("The check %s was not found. Use lgp check --list" % str(err))
         return self.checklist
 
     def start_checks(self):
         for func in self.get_checklist():
-            self.logger = logging.getLogger(func.__name__)
+            loggername = func.__name__
+            loggername = loggername.replace('_',':', 1)
+            self.logger = logging.getLogger(loggername)
+
             result = func(self)
             # result possible values:
             #   negative -> error occured !
@@ -259,13 +258,14 @@ class Checker(SetupInfo):
             elif result>0:
                 self.counter += 1
 
+    # TODO dump with --help and drop the command-line option
     def list_checks(self):
         all_checks = self.get_checklist(all=True)
         checks     = self.get_checklist()
         if len(checks)==0:
             print "No available check."
         else:
-            print "You can use the --exclude or --include options\n"
+            print "You can use the --set, --exclude or --include options\n"
             msg = "Current active checks"
             print msg; print len(msg) * '='
             for check in checks:
@@ -280,9 +280,11 @@ class Checker(SetupInfo):
 # ========================================================
 #
 # Check functions collection starts here
+# TODO make a package to add easily external checkers
+# TODO instead of OK/NOK
 #
-# IMPORTANT ! all checkers have to return a valid status !
-# Example: OK or NOK
+# IMPORTANT ! all checkers should return a valid status !
+# Example: OK, NOK or None
 #
 # ========================================================
 def check_keyrings(checker):
@@ -300,7 +302,7 @@ def check_pydistutils(checker):
     return OK
 
 def check_builder(checker):
-    """check if the builder is correct """
+    """check if the builder has been changed"""
     debuilder = os.environ.get('DEBUILDER') or False
     if debuilder:
         checker.logger.warn('you have set a different builder in DEBUILDER. Unset it if in doubt')
@@ -308,54 +310,88 @@ def check_builder(checker):
 
 def check_debian_dir(checker):
     """check the debian* directory """
+    checker.current_distrib = "unstable"
     debian_dir = checker.get_debian_dir()
-    return os.path.isdir(debian_dir)
+    return isdir(debian_dir)
 
 def check_debian_rules(checker):
     """check the debian*/rules file (filemode) """
     debian_dir = checker.get_debian_dir()
     status = OK
-    status = status and os.path.isfile(debian_dir + '/rules')
-    status = status and is_executable(debian_dir + '/rules')
+    status = status and isfile(os.path.join(debian_dir, 'rules'))
+    status = status and is_executable(os.path.join(debian_dir, 'rules'))
     return status
 
 def check_debian_copying(checker):
-    """check debian*/copyright file """
+    """check debian*/copyright file"""
     debian_dir = checker.get_debian_dir()
-    return os.path.isfile(os.path.join(debian_dir,'copyright'))
+    return isfile(os.path.join(debian_dir,'copyright'))
+
+def check_debian_source_value(checker):
+    """check debian source field value"""
+    upstream_name = checker.get_upstream_name()
+    debian_name   = checker.get_debian_name()
+    if upstream_name != debian_name:
+        checker.logger.warn("upstream project name (%s) is different from the "
+                            "Source filed value in your debian/control (%s)"
+                            % (upstream_name, debian_name))
+    return OK
+
 
 def check_debian_changelog(checker):
-    """your debian changelog is not parsable"""
+    """your debian changelog contains error(s)"""
     debian_dir = checker.get_debian_dir()
-    CHANGELOG = os.path.join(debian_dir, '/changelog')
-    status= OK
-    if os.path.isfile(CHANGELOG):
-        cmd = "sed -ne '/UNRELEASED/p' debian/changelog"
+    CHANGELOG = os.path.join(debian_dir, 'changelog')
+    status = OK
+    if isfile(CHANGELOG):
+        cmd = "sed -ne '/UNRELEASED/p' %s" % CHANGELOG
         _, output = commands.getstatusoutput(cmd)
         if output:
             status = NOK
-            checker.logger.error('UNRELEASED distribution(s) in debian changelog:')
-            print output
-        cmd = "sed -ne '/DISTRIBUTION/p' debian/changelog"
+            checker.logger.error('UNRELEASED keyword in debian changelog')
+        cmd = "sed -ne '/DISTRIBUTION/p' %s" % CHANGELOG
         _, output = commands.getstatusoutput(cmd)
         if output:
-            status = NOK
-            checker.logger.warn('You can now use the default "unstable" string in your debian changelog:')
-            print output
-        cmd = "dpkg-parsechangelog"
+            checker.logger.info('Default distribution value should be "unstable" in your debian changelog')
+        cmd = "dpkg-parsechangelog >/dev/null"
         _, output = commands.getstatusoutput(cmd)
         if output:
             status = NOK
+            checker.logger.error(output)
+    return status
+
+def check_debian_maintainer(checker):
+    """check Maintainer field in debian/control file"""
+    status = OK
+    cmd = "grep '^Maintainer' debian/control | cut -d' ' -f2- | tr -d '\n'"
+    cmdstatus, output = commands.getstatusoutput(cmd)
+    if output.strip() != 'Logilab S.A. <contact@logilab.fr>':
+        checker.logger.info("Maintainer value can be 'Logilab S.A. <contact@logilab.fr>'")
+    return status
+
+def check_debian_uploader(checker):
+    """check Uploaders field in debian/control file"""
+    status = OK
+    cmd = "dpkg-parsechangelog | grep '^Maintainer' | cut -d' ' -f2- | tr -d '\n'"
+    _, output = commands.getstatusoutput(cmd)
+    cmd = 'grep "%s" debian/control' % output
+    cmdstatus, _ = commands.getstatusoutput(cmd)
+    if cmdstatus:
+        # FIXME
+        #checker.logger.error("'%s' is not found in Uploaders field" % output)
+        #status = NOK
+        checker.logger.warn("'%s' is not found in Uploaders field" % output)
+        checker.logger.warn(check_debian_uploader.__doc__)
     return status
 
 def check_readme(checker):
-    """check the upstream README file """
+    """the upstream README file is missing"""
     if not isfile('README'):
         checker.logger.warn(check_readme.__doc__)
     return OK
 
 def check_changelog(checker):
-    """check the upstream ChangeLog """
+    """the upstream ChangeLog file is missing"""
     status = OK
     if not isfile(CHANGEFILE):
         checker.logger.warn(check_changelog.__doc__)
@@ -368,19 +404,21 @@ def check_changelog(checker):
 
 def check_copying(checker):
     """check upstream COPYING file """
-    if not os.path.isfile('COPYING'):
+    if not isfile('COPYING'):
         checker.logger.warn(check_copying.__doc__)
     return OK
 
 def check_tests_directory(checker):
-    """check the tests? directory """
-    return isdir('test') or isdir('tests')
+    """check your tests? directory """
+    if not (isdir('test') or isdir('tests')):
+        checker.logger.warn(check_copying.__doc__)
+    return OK
 
 def check_run_tests(checker):
     """run the unit tests """
     testdirs = ('test', 'tests')
     for testdir in testdirs:
-        if os.path.isdir(testdir):
+        if isdir(testdir):
             cond_exec('pytest', confirm=True, retry=True)
             break
     return OK
@@ -390,11 +428,15 @@ def check_setup_file(checker):
     return isfile('setup.py') or isfile('setup.mk')
 
 def check_makefile(checker):
-    """check makefile file and dependencies """
+    """check makefile file and expected targets (project, version)"""
     status = OK
-    status = status and os.path.isfile("setup.mk")
-    # FIXME
-    #status = status and _check_make_dependencies()
+    setup_file = checker.config.setup_file
+    status = status and isfile(setup_file)
+    for cmd in ['%s project', '%s version']:
+        cmd %= setup_file
+        if not subprocess.call(cmd.split()):
+            checker.logger.error("%s not a valid command" % cmd)
+        status = NOK
     return status
 
 def check_homepage(checker):
@@ -416,6 +458,7 @@ def check_announce(checker):
 
 def check_bin(checker):
     """check executable script files in bin/ """
+    BASE_EXCLUDE = ('CVS', '.svn', '.hg', 'bzr')
     status = OK
     if not exists('bin/'):
         return status
@@ -438,10 +481,10 @@ def check_bin(checker):
 def check_documentation(checker):
     """check project's documentation"""
     status = OK
-    if os.path.isdir('doc'):
+    if isdir('doc'):
         # FIXME
         # should be a clean target in setup.mk for example
-        # and os.path.isfile('doc/Makefile') or os.path.isfile('doc/makefile'):
+        # and isfile('doc/Makefile') or isfile('doc/makefile'):
         #if confirm('build documentation ?'):
         #os.chdir('doc')
         #status = cond_exec('make', retry=True)
@@ -453,52 +496,36 @@ def check_documentation(checker):
 def check_repository(checker):
     """check repository status (not up-to-date) """
     try:
+        from logilab.devtools.vcslib import get_vcs_agent
         vcs_agent = get_vcs_agent(checker.config.pkg_dir)
-        result = vcs_agent.not_up_to_date(checker.config.pkg_dir)
-        if result:
-            checker.logger.warn("vcs_agent returns: %s" % result)
-            return NOK
+        if vcs_agent:
+            result = vcs_agent.not_up_to_date(checker.config.pkg_dir)
+            if result:
+                checker.logger.warn("vcs_agent returns:\n%s" % pformat(result))
+                return NOK
+    except ImportError:
+        checker.logger.warn("you need to install logilab vcslib package for this check")
     except NotImplementedError:
         checker.logger.warn("the current vcs agent isn't yet supported")
     return OK
 
 def check_release_number(checker):
-    """check inconsistency with release number """
+    """check the versions coherence between upstream and debian/changelog"""
     status = OK
-    if hasattr(checker, "_package") and checker._package_format == "pkginfo":
-        pi = checker._package
-    else:
-        return status
-    dirname = checker.config.pkg_dir
-    version = normalize_version(pi.version)
-    try:
-        cl_version = ChangeLog(find_ChangeLog(dirname)).get_latest_revision()
-        cl_version = normalize_version(str(cl_version))
-        if cl_version != version:
-            msg = 'version inconsistency : found %s in ChangeLog \
-(reference is %s)'
-            checker.logger.error(msg % (cl_version, version))
-            status = NOK
-    except ChangeLogNotFound:
-        checker.logger.warn('upstream %s was not found' % CHANGEFILE)
-
-    deb_version = pi.debian_version()
-    deb_version = normalize_version(deb_version.split('-', 1)[0])
-    if deb_version != version:
-        msg = 'version inconsistency : found %s in debian/changelog \
-(reference is %s)'
-        checker.logger.error('debian/changelog', None,
-                       msg % (deb_version, version))
+    try: 
+        checker.compare_versions()
+    except LGPException, err:
+        checker.logger.critical(err)
         status = NOK
     return status
 
 def check_manifest_in(checker):
-    """check MANIFEST.in content"""
+    """to correct unmatched files, please include or exclude them in MANIFEST.in"""
     status = OK
     dirname = checker.config.pkg_dir
     absfile = join(dirname, 'MANIFEST.in')
     # return immediatly if no file available
-    if not os.path.isfile(absfile):
+    if not isfile(absfile):
         return status
 
     # check matched files
@@ -509,8 +536,11 @@ def check_manifest_in(checker):
             i = matched.index(path)
             matched.pop(i)
         except ValueError:
-            checker.logger.warn('%s is not matched' % path)
-            status = NOK
+            checker.logger.warn('%s unmatched' % path)
+            # FIXME keep valid status till ``#2888: lgp check ignore manifest # "prune"``
+            # path command not resolved
+            # See http://www.logilab.org/ticket/2888
+            #status = NOK
     # check garbage
     for filename in matched:
         if match_extensions(filename, JUNK_EXTENSIONS):
@@ -518,7 +548,7 @@ def check_manifest_in(checker):
     return status
 
 def check_include_dirs(checker):
-    """check include_dirs"""
+    """check include_dirs declared in setup file"""
     if hasattr(checker, "_package") and hasattr(checker._package, 'include_dirs'):
         for directory in checker._package.include_dirs:
             if not exists(directory):
@@ -528,13 +558,14 @@ def check_include_dirs(checker):
     return OK
 
 def check_debsign(checker):
-    """add the DEBSIGN_KEYID environment variable to sign directly """
+    """Hint: you can add DEBSIGN_KEYID to your environment and use a gpg-agent to sign directly"""
     if 'DEBSIGN_KEYID' not in os.environ:
-        checker.logger.info(check_debsign.__doc__)
+        logging.info(check_debsign.__doc__)
+        return
     return OK
 
 def check_scripts(checker):
-    """check declared scripts"""
+    """check scripts declared in setup file"""
     if hasattr(checker, "_package") and hasattr(checker._package, 'scripts'):
         detected_scripts = get_default_scripts(checker._package)
         scripts = getattr(checker._package, 'scripts', [])
@@ -545,9 +576,9 @@ def check_scripts(checker):
     return OK
 
 def check_package_info(checker):
-    """check package information """
+    """check package information"""
     status = OK
-    if hasattr(checker, "_package") and checker._package_format == "pkginfo":
+    if hasattr(checker, "_package") and checker.package_format == "PackageInfo":
         pi = checker._package
     else:
         return status
@@ -592,11 +623,12 @@ def check_package_info(checker):
 # ===============================
 
 def check_shebang(checker):
-    """check #! signature for shell and python script """
+    """check #! signature for shell and python script (not implemented)"""
+    # TODO make a test with file utility and check #!/bin/... or #!/usr/bin/env python
     raise NotImplementedError("use best practises")
 
 def check_deprecated(checker):
-    """check attributes in deprecation"""
+    """check attributes in deprecation (not implemented)"""
     raise NotImplementedError("use right pylint options")
 
 def check_pylint(checker):
