@@ -22,6 +22,7 @@ import glob
 import sys
 from os.path import basename, join, split, exists
 from subprocess import Popen, PIPE
+import logging
 
 from logilab.devtools.lgp.exceptions import (ArchitectureException,
                                              DistributionException)
@@ -33,11 +34,15 @@ PUBLIC_RGX = re.compile('PUBLIC\s+"-//(?P<group>.*)//DTD (?P<pubid>.*)//(?P<lang
 
 # The known distribution are depending on the pbuilder setup in /opt/buildd
 # Find a way to retrieve dynamically
-KNOWN_DISTRIBUTIONS = {'etch': 'stable',
+# f = email.message_from_file(file('/usr/share/cdebootstrap/suites'))
+# [i.split() for i in f.get_payload().split('\n\n') if i]
+KNOWN_DISTRIBUTIONS = {'etch': 'oldstable',
+                       'oldstable': 'oldstable',
+                       'lenny': 'stable',
                        'stable': 'stable',
                        'unstable': 'unstable',
                        'sid': 'unstable',
-                       'lenny': 'testing',
+                       'squeeze': 'testing',
                        'testing': 'testing',
                        'feisty':'feisty',
                        'gutsy':'gutsy',
@@ -45,8 +50,10 @@ KNOWN_DISTRIBUTIONS = {'etch': 'stable',
                        'intrepid':'intrepid',
                        'jaunty':'jaunty',
                        'dapper':'dapper',
+                       'experimental': 'unstable',
                       }
 
+
 class SGMLCatalog:
     """ handle SGML catalog information
     i.e. map dtds to their public id
@@ -145,36 +152,38 @@ def cond_exec(cmd, confirm=False, retry=False):
         else:
             return False
 
-def get_distributions(distrib=None):
+def get_distributions(distrib=None, basetgz=None):
     """ensure that the target distributions exist or return all the valid distributions
     """
-    if distrib is None:
+    distributions = KNOWN_DISTRIBUTIONS
+    if distrib is None or distrib == 'known':
         distrib = KNOWN_DISTRIBUTIONS.keys()
         return tuple(set(distrib))
     elif distrib == 'target':
         distrib = KNOWN_DISTRIBUTIONS.values()
         return tuple(set(distrib))
-    elif distrib == 'known':
-        distrib = KNOWN_DISTRIBUTIONS
-        return tuple(set(distrib))
-    elif distrib == 'all':
-        directories = glob.glob(join(os.getcwd(), "debian.*"))
-        distrib = [basename(d).split('.')[1] for d in directories]
-        # 'unstable' distribution should be always present
-        distrib.append('unstable')
+    elif 'all' in distrib:
+        # this case fixes unittest_distributions.py when basetgz is None
+        if basetgz is None:
+            return set(KNOWN_DISTRIBUTIONS)
+        distrib = [os.path.basename(f).split('-', 1)[0]
+                   for f in glob.glob(os.path.join(basetgz,'*.tgz'))]
+        return set(KNOWN_DISTRIBUTIONS) & set(distrib)
 
     mapped = ()
-    if type(distrib) is str:
-        distrib = distrib.split(',')
+    # check and filter if known
     for t in distrib:
-        if t not in KNOWN_DISTRIBUTIONS:
+        if (len(sys.argv)>1 and sys.argv[1] not in ["setup"]):
+            distributions = get_distributions('all', basetgz)
+        if t not in distributions:
+            logging.critical("'%s' image not found in '%s'" % (t, basetgz))
             raise DistributionException(t)
-        mapped += (KNOWN_DISTRIBUTIONS[t],)
+        mapped += (t,)
     distrib = mapped
+
     return tuple(set(distrib))
 
-
-def get_architectures(archi="current"):
+def get_architectures(archi=None):
     """ Ensure that the architectures exist
 
         :param:
@@ -184,14 +193,26 @@ def get_architectures(archi="current"):
             list of architecture
     """
     known_archi = Popen(["dpkg-architecture", "-L"], stdout=PIPE).communicate()[0].split()
-    if archi == "current":
+    if archi is None or archi == ["current"]:
         archi = Popen(["dpkg", "--print-architecture"], stdout=PIPE).communicate()[0].split()
     else:
-        if archi == "all":
+        if archi == ["all"]:
             return archi
-        if type(archi) is str:
-            archi = archi.split(',')
         for a in archi:
             if a not in known_archi:
                 raise ArchitectureException(a)
     return archi
+
+def cached(func):
+    """run a function only once and return always the same cache
+
+       This decorator is used to reduce the overheads due to many system calls
+    """
+    def decorated(*args, **kwargs):
+        try:
+            return decorated._once_result
+        except AttributeError:
+            decorated._once_result = func(*args, **kwargs)
+            return decorated._once_result
+    return decorated
+
